==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_base/lib/corelibs/superset/dff.c <==
/*

NAME:
   sep_put_distrib_format_tag -- put ...

SYNOPSIS
   #include <sep3d.h>

   int sep_put_distrib_format_tag(char *tag_history, char *tag_dff)

PARAMETER
   INPUT
   char *tag_history   :  tag of History File
   char *tag_dff   :  tag to Header Format File

RETURN VALUE
    0 if successful
   -1 if fails for other reasons

DESCRIPTION
   Initialize the History File tag_history as a Sep3d History File.
   Initialize the Header Format File tag_dff as a Sep3d Header 
   Format File. Link tag_dff to tag_history.


KEYWORDS
   header 

SEE ALSO
   superset

AUTHOR
   Bob Clapp Jan ... 2003

*/
#include <sep3d.h>
#include<unistd.h>
#include "./../include/streamlist.h"

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep_put_distrib_format_tag(char* tag_history, char* tag_dff)
_XFUNCPROTOEND
#else
int sep_put_distrib_format_tag(tag_history, tag_dff)
char *tag_history; 
char *tag_dff;
#endif
{
    streaminf *info;


    /* Initialize history file as Sep3d history file */
    info = tag_info(tag_history, TAG_INQUIRE);

    info->distribformatfile = (char *) malloc(1+(int)strlen(tag_dff));
    strcpy(info->distribformatfile,tag_dff);
    auxputch("dff","s", tag_dff, tag_history);
   if(0==strcmp(tag_dff,"-1")) return(0);

    /* Open Grid Format File file */
 
    switch( info->entrytype ){
      case( STREAMIN ):
				seperr("Can not call sep_put_distrib_format_tag for STREAMIN tag \n");
				break;
      case( STREAMOUT ):
/* Notice that is auxscr to allow routines to get information
on header keys also if it is a Header Format File linked with an output 
History File */
	auxout(tag_dff);
	if(1==sep_tag_is_pipe(tag_history)){
    /*if we are dealing with stodut tag we want this file to
     disapear at the end of program execution */
    unlink(tag_dff);
	}
	
	  break;
      case( STREAMINOUT ):
	auxscr(tag_dff);
	break;
      case( STREAMSCR ):
	auxscr(tag_dff);
	break;
    }    


    return 0;
}
/*<
sep_get_distrib_format_tag 

   
USAGE 
int sep_get_distrib_format_tag(tag_history,tag_distrib)

INPUT PARAMETER
   char-*tag_history    tag of History File

OUTPUT PARAMETER
   char-**tag_distrib     tag to Header Format File

RETURN VALUE
   -1= if tag_history is not a SEP History File
    0= if successful
   +2= if tag_history has no Grid Format File associated with it.

DESCRIPTION
   Get tag for Header Format file pointed by tag_history.

CATEGORY
Lib:Sep3d:Grid access

COMPILE LEVEL
DISTR
>*/
/*
KEYWORDS
   distrib 

SEE ALSO
   sep3d

AUTHOR

*/

/* 
*
* LOGIC: 
* Determine if history tag points to an SEP history file
* check if distrib tag is already specified in sepinfo
* case +: write tag in sepinfo to tag_distrib
* case -:
*        if file is used for IN or INOUT 
*            check for tag_distrib in history file
*                  case +: copy it 
*                  case -: is a SEP77 History file
*        else
*            check if distrib tag is written on command line
*            if not create a default distrib tag 
*                  tag_distrib = tag_history + "^^"
*
*/

#include "sep3d.h"
#include "./../include/streamlist.h"
#include "./../include/sep_main_internal.h"
#include<time.h>


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep_get_distrib_format_tag(char* tag_history, char** tag_distrib)
_XFUNCPROTOEND
#else
int sep_get_distrib_format_tag(tag_history, tag_distrib)
char *tag_history; 
char **tag_distrib;
#endif
{
    streaminf *info;
    char tag_distrib_buf[1024],scratch_file[1024];
	  int test=0,put_end,found=0;
     time_t tempt;

		

		/*changed so the first call can be SEP3d, might not be correct ...*/
    info = tag_info(tag_history, TAG_INQUIRE);  /* get info on this tag */
    if(info == SEPPOINTNULL) return 1;   /* Not a an SEP History File */




 if((info->distribformatfile) == SEPSTRNULL) {
   if((info->entrytype == STREAMINOUT)){
       if(auxpar("dff","s", tag_distrib_buf, tag_history) == 1){
         found=1;
         *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(*tag_distrib, tag_distrib_buf);
         info->distribformatfile = (char *)malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(info->distribformatfile, *tag_distrib);
        }
     }

    if((info->entrytype == STREAMIN) && found==0){
       if(auxpar("dff","s", tag_distrib_buf, tag_history) == 1) test=1;
       if(getch("dff_in","s", tag_distrib_buf) == 1) test=1;
       if(test == 0 || 0==strncmp("-1",tag_distrib_buf, 2)){
          return 2;     /* is a simple SEP77 history file */
				}
       else { /* tag is not in sepinfo but it is in history file,
            write it into sepinfo.                       */
  			/*This is a new error condition to force people to call init_3d at
				   the begining of any sep3d program */
				  if(0==strcmp("in",tag_history))
			    seperr("sep_get_distrib_tag():must call init_3d at the begining of your sep3d program \n");

         *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(*tag_distrib, tag_distrib_buf);
         info->distribformatfile = (char *)malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(info->distribformatfile, *tag_distrib);
         /* open GFF appropriately */
         auxin(*tag_distrib);
         return 0;
       }
      }
      else if(found==0){
        /*check for distrib_format_file being overwritten on command line */
        if(!getch("dff","s", tag_distrib_buf)) {
          /* for the special case of stdout info->distribname = stdout
          so we must getch the name from the command line */
          if(strcmp(info->tagname, "out") == 0){
            if ( info->isapipe || isapipe(fileno(info->headfile)) ){

              /*pt1 of 2 of lame trick.  Basically we are going to create
              a temporary dff file which we unlink from.  When sep3d_close
              is called the contents of the file are copied down a socket.
              Far from elegant, but it should work. Logic is otherwise
              we will have to create temp buffers for the dff file because
              we can't open socket until the history file is sent with
              the correct socket # info */

              /*first create the temporary file name */
              datapath(scratch_file);
              time(&tempt);
              srand(tempt);
              put_end=rand();
              sprintf(tag_distrib_buf,"%s%s%d",scratch_file,"dff_",put_end);
            }

            else if(findnm(fileno(info->headfile), tag_distrib_buf, sizeof(tag_distrib_buf)) ==0){
              seperr("When stdout is not in the same directory you need to specify the Grid Format file with dff on the command line") ;
            }
          }
         /* Doesn't account for auxilary stuff directed along pipes  */
          else  strcpy(tag_distrib_buf, info->headername);
          strcat(tag_distrib_buf,"@@@@@@");
        }
        *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
        strcpy(*tag_distrib, tag_distrib_buf);
        sep_put_distrib_format_tag(tag_history, tag_distrib_buf);
        return 0;
     }
    }
    else { /* sepinfo contains tag_distrib, so just copy it */
      *tag_distrib= (char *) malloc(1+(int)strlen(info->distribformatfile));
      strcpy(*tag_distrib,info->distribformatfile);
       if(0==strncmp("-1",*tag_distrib, 2)){ return 2;}
      	else{ return 0;}
    }

}


/*<
fget_distrib_format_tag

USAGE
 int fget_distrib_format_tag (tag_history,tag_distrib)

INPUT PARAMETERS
  tag_history - char*   tag of history file

 OUTPUT PARAMETERS
  tag_distrib - char*   tag of distrib  file


DESCRIPTION
  Allows fortran to access tag of distrib  file (could be written in a better way
  that is totally transparent, but don't want to bother for now)

CATEGORY
Lib:Sep3d:Grid access

COMPILE LEVEL
DISTR
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int fget_distrib_format_tag(char * tag_history, char *tag_distrib)
_XFUNCPROTOEND
#else
int fget_distrib_format_tag(tag_history, tag_distrib)
char * tag_history;
char *tag_distrib;
#endif
{
char *temp[1];
int ierr;

ierr=sep_get_distrib_format_tag(tag_history,temp);
if(ierr==0) {
  strcpy(tag_distrib,*temp);
  free(*temp);
}

return ierr;
}

==> seplib-6.3.8.tar.gz: seplib-6.3.8/seplib_base/lib/corelibs/superset/dff.c <==
/*

NAME:
   sep_put_distrib_format_tag -- put ...

SYNOPSIS
   #include <sep3d.h>

   int sep_put_distrib_format_tag(char *tag_history, char *tag_dff)

PARAMETER
   INPUT
   char *tag_history   :  tag of History File
   char *tag_dff   :  tag to Header Format File

RETURN VALUE
    0 if successful
   -1 if fails for other reasons

DESCRIPTION
   Initialize the History File tag_history as a Sep3d History File.
   Initialize the Header Format File tag_dff as a Sep3d Header 
   Format File. Link tag_dff to tag_history.


KEYWORDS
   header 

SEE ALSO
   superset

AUTHOR
   Bob Clapp Jan ... 2003

*/
#include <sep3d.h>
#include<unistd.h>
#include "streamlist.h"

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep_put_distrib_format_tag(char* tag_history, char* tag_dff)
_XFUNCPROTOEND
#else
int sep_put_distrib_format_tag(tag_history, tag_dff)
char *tag_history; 
char *tag_dff;
#endif
{
    streaminf *info;


    /* Initialize history file as Sep3d history file */
    info = tag_info(tag_history, TAG_INQUIRE);

    info->distribformatfile = (char *) malloc(1+(int)strlen(tag_dff));
    strcpy(info->distribformatfile,tag_dff);
    auxputch("dff","s", tag_dff, tag_history);
   if(0==strcmp(tag_dff,"-1")) return(0);

    /* Open Grid Format File file */
 
    switch( info->entrytype ){
      case( STREAMIN ):
				seperr("Can not call sep_put_distrib_format_tag for STREAMIN tag \n");
				break;
      case( STREAMOUT ):
/* Notice that is auxscr to allow routines to get information
on header keys also if it is a Header Format File linked with an output 
History File */
	auxout(tag_dff);
	if(1==sep_tag_is_pipe(tag_history)){
    /*if we are dealing with stodut tag we want this file to
     disapear at the end of program execution */
    unlink(tag_dff);
	}
	
	  break;
      case( STREAMINOUT ):
	auxscr(tag_dff);
	break;
      case( STREAMSCR ):
	auxscr(tag_dff);
	break;
    }    


    return 0;
}
/*<
sep_get_distrib_format_tag 

   
USAGE 
int sep_get_distrib_format_tag(tag_history,tag_distrib)

INPUT PARAMETER
   char-*tag_history    tag of History File

OUTPUT PARAMETER
   char-**tag_distrib     tag to Header Format File

RETURN VALUE
   -1= if tag_history is not a SEP History File
    0= if successful
   +2= if tag_history has no Grid Format File associated with it.

DESCRIPTION
   Get tag for Header Format file pointed by tag_history.

CATEGORY
Lib:Sep3d:Grid access

COMPILE LEVEL
DISTR
>*/
/*
KEYWORDS
   distrib 

SEE ALSO
   sep3d

AUTHOR

*/

/* 
*
* LOGIC: 
* Determine if history tag points to an SEP history file
* check if distrib tag is already specified in sepinfo
* case +: write tag in sepinfo to tag_distrib
* case -:
*        if file is used for IN or INOUT 
*            check for tag_distrib in history file
*                  case +: copy it 
*                  case -: is a SEP77 History file
*        else
*            check if distrib tag is written on command line
*            if not create a default distrib tag 
*                  tag_distrib = tag_history + "^^"
*
*/

#include "sep3d.h"
#include "streamlist.h"
#include "sep_main_internal.h"
#include<time.h>


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep_get_distrib_format_tag(char* tag_history, char** tag_distrib)
_XFUNCPROTOEND
#else
int sep_get_distrib_format_tag(tag_history, tag_distrib)
char *tag_history; 
char **tag_distrib;
#endif
{
    streaminf *info;
    char tag_distrib_buf[1024],scratch_file[1024];
	  int test=0,put_end,found=0;
     time_t tempt;

		

		/*changed so the first call can be SEP3d, might not be correct ...*/
    info = tag_info(tag_history, TAG_INQUIRE);  /* get info on this tag */
    if(info == SEPPOINTNULL) return 1;   /* Not a an SEP History File */




 if((info->distribformatfile) == SEPSTRNULL) {
   if((info->entrytype == STREAMINOUT)){
       if(auxpar("dff","s", tag_distrib_buf, tag_history) == 1){
         found=1;
         *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(*tag_distrib, tag_distrib_buf);
         info->distribformatfile = (char *)malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(info->distribformatfile, *tag_distrib);
        }
     }

    if((info->entrytype == STREAMIN) && found==0){
       if(auxpar("dff","s", tag_distrib_buf, tag_history) == 1) test=1;
       if(getch("dff_in","s", tag_distrib_buf) == 1) test=1;
       if(test == 0 || 0==strncmp("-1",tag_distrib_buf, 2)){
          return 2;     /* is a simple SEP77 history file */
				}
       else { /* tag is not in sepinfo but it is in history file,
            write it into sepinfo.                       */
  			/*This is a new error condition to force people to call init_3d at
				   the begining of any sep3d program */
				  if(0==strcmp("in",tag_history))
			    seperr("sep_get_distrib_tag():must call init_3d at the begining of your sep3d program \n");

         *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(*tag_distrib, tag_distrib_buf);
         info->distribformatfile = (char *)malloc(1+(int)strlen(tag_distrib_buf));
         strcpy(info->distribformatfile, *tag_distrib);
         /* open GFF appropriately */
         auxin(*tag_distrib);
         return 0;
       }
      }
      else if(found==0){
        /*check for distrib_format_file being overwritten on command line */
        if(!getch("dff","s", tag_distrib_buf)) {
          /* for the special case of stdout info->distribname = stdout
          so we must getch the name from the command line */
          if(strcmp(info->tagname, "out") == 0){
            if ( info->isapipe || isapipe(fileno(info->headfile)) ){

              /*pt1 of 2 of lame trick.  Basically we are going to create
              a temporary dff file which we unlink from.  When sep3d_close
              is called the contents of the file are copied down a socket.
              Far from elegant, but it should work. Logic is otherwise
              we will have to create temp buffers for the dff file because
              we can't open socket until the history file is sent with
              the correct socket # info */

              /*first create the temporary file name */
              datapath(scratch_file);
              time(&tempt);
              srand(tempt);
              put_end=rand();
              sprintf(tag_distrib_buf,"%s%s%d",scratch_file,"dff_",put_end);
            }

            else if(findnm(fileno(info->headfile), tag_distrib_buf, sizeof(tag_distrib_buf)) ==0){
              seperr("When stdout is not in the same directory you need to specify the Grid Format file with dff on the command line") ;
            }
          }
         /* Doesn't account for auxilary stuff directed along pipes  */
          else  strcpy(tag_distrib_buf, info->headername);
          strcat(tag_distrib_buf,"@@@@@@");
        }
        *tag_distrib= (char *) malloc(1+(int)strlen(tag_distrib_buf));
        strcpy(*tag_distrib, tag_distrib_buf);
        sep_put_distrib_format_tag(tag_history, tag_distrib_buf);
        return 0;
     }
    }
    else { /* sepinfo contains tag_distrib, so just copy it */
      *tag_distrib= (char *) malloc(1+(int)strlen(info->distribformatfile));
      strcpy(*tag_distrib,info->distribformatfile);
       if(0==strncmp("-1",*tag_distrib, 2)){ return 2;}
      	else{ return 0;}
    }

}


/*<
fget_distrib_format_tag

USAGE
 int fget_distrib_format_tag (tag_history,tag_distrib)

INPUT PARAMETERS
  tag_history - char*   tag of history file

 OUTPUT PARAMETERS
  tag_distrib - char*   tag of distrib  file


DESCRIPTION
  Allows fortran to access tag of distrib  file (could be written in a better way
  that is totally transparent, but don't want to bother for now)

CATEGORY
Lib:Sep3d:Grid access

COMPILE LEVEL
DISTR
>*/

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int fget_distrib_format_tag(char * tag_history, char *tag_distrib)
_XFUNCPROTOEND
#else
int fget_distrib_format_tag(tag_history, tag_distrib)
char * tag_history;
char *tag_distrib;
#endif
{
char *temp[1];
int ierr;

ierr=sep_get_distrib_format_tag(tag_history,temp);
if(ierr==0) {
  strcpy(tag_distrib,*temp);
  free(*temp);
}

return ierr;
}
